//************************************************************************
//*                          BipDATE Win V0.9.1                          *
//*----------------------------------------------------------------------*
//*                  BipDATE Class for Microsoft Windows                 *
//*                                                                      *
//*                                                         P. Basset    *
//************************************************************************


#include <sstream>
#include <time.h>
#include "BipDATE.h"


// error codes for BP_Date::iBPError (also return by BP_Date::geterror() )
//
//  0 = non error
//  1 = day is bad
//  2 = month is bad
//  3 = year is bad
//  4 = hour is bad
//  5 = minute is bad
//  6 = second is bad
//  7 = input date is bad
//  8 = date is empty
//  9 = format is empty



//-----------------------------------------------------------------
//!    BP_Date::BP_Date : class constructor                       !
//-----------------------------------------------------------------
//! return : none                                                 !
//-----------------------------------------------------------------
BP_Date::BP_Date ()
{
	// reset date
	tBP_Date = 0;
	bBP_Null = true;

	// reset format
	sBP_Format = "";

	// reset error code
	iBPError = 0;

}



//-----------------------------------------------------------------
//!    BP_Date::~BP_Date : class destructor                       !
//-----------------------------------------------------------------
//! return : none                                                 !
//-----------------------------------------------------------------
BP_Date::~BP_Date ()
{
}



//-----------------------------------------------------------------
//!    BP_Date::reset : reset date and format                     !
//-----------------------------------------------------------------
//! return : none                                                 !
//-----------------------------------------------------------------
//! rem : date is set to null and format to ""                    !
//-----------------------------------------------------------------
BP_Date::reset (void)
{
	// set date to 0
	tBP_Date = 0;
	bBP_Null = true;

	// format is null
	sBP_Format = "";

	// reset error code
	iBPError = 0;

}



//-----------------------------------------------------------------
//!    BP_Date::setdatetime : fill date and time into a BP_date   !
//!                           container                           !
//-----------------------------------------------------------------
//! BPyear : year (min 1970, max 2038)                            !
//! BPmonth : month (min 1, max 12)                               !
//! BPday : day (min 1, max 31)                                   !
//! BPhour : hour (min 0, max 23)                                 !
//! BPmin : minute (min 0, max 59)                                !
//! BPsec : second (min 0, max 59)                                !
//!																  !
//! return : none                                                 !
//-----------------------------------------------------------------
//! if error occurs, the date is set to null and                  !
//!     BP_Date::geterror() return error code                     !
//-----------------------------------------------------------------
//! minimum date : 1 january 1970  0 hour 0 minute 0 second       !
//! maximum date : 19 january 2038  3 hours 14 minutes 7 seconds  !
//! rem : the min and max hour depends on the GMT+- local time    !
//-----------------------------------------------------------------
BP_Date::setdatetime (const int BPyear, const int BPmonth, const int BPday,
			  const int BPhour, const int BPmin, const int BPsec)
{

	tm tmBPStruct;
	
	// reset error code
	iBPError = 0;

	// verify input values
	if (BPday < 1 || BPday > 31)
		iBPError = 1;

	if (BPmonth < 1 || BPmonth > 12)
		iBPError = 2;

	if (BPyear < 1970 || BPyear > 2038)
		iBPError = 3;

	if (BPhour < 0 || BPhour > 24)
		iBPError = 4;

	if (BPmin < 0 || BPmin > 59)
		iBPError = 5;

	if (BPsec < 0 || BPsec > 59)
		iBPError = 6;

	// if ok, store date and time
	if (iBPError == 0)
	{
		tmBPStruct.tm_sec = BPsec;
		tmBPStruct.tm_min = BPmin;
		tmBPStruct.tm_hour = BPhour;
		tmBPStruct.tm_mday = BPday;
		tmBPStruct.tm_mon = BPmonth - 1;
		tmBPStruct.tm_year = BPyear - 1900;
		tmBPStruct.tm_wday = 0;
		tmBPStruct.tm_yday = 0;
		tmBPStruct.tm_isdst = 0;

		BP_Date::tBP_Date = mktime (&tmBPStruct);

		// date is bad
		if (tBP_Date == (time_t)-1)
		{
			iBPError = 7;

			// set date to null
			BP_Date::setnull ();
		}
		else
			// date  is not null
			bBP_Null = false;


	}
	else
	{
		// set date to null
		BP_Date::setnull ();
	}

}



//-----------------------------------------------------------------
//!    BP_Date::set : fill formated date and time into a BP_date  !
//!                           container                           !
//-----------------------------------------------------------------
//! BPdate : formated date and/or time                            !
//! rem : the formated date/time must have the same order than    !
//!    the format. the numeric values must be separated by a non  !
//!    numeric separator.                                         !
//!																  !
//! return : none                                                 !
//-----------------------------------------------------------------
//! if error occurs, the date is set to null and                  !
//!     BP_Date::geterror() return error code                     !
//-----------------------------------------------------------------
//! see BP_Date::setdatetime for max and min date                 !
//-----------------------------------------------------------------
//! warning : BP_Date::set try to detect the content of the input !
//!    string, but some format can fail,                          !
//!    BP_Date::setdatetime is more secure                        !
//-----------------------------------------------------------------
BP_Date::set (const std::string BPdate)
{
	int BPyear;
	int BPmonth;
	int BPday;
	int BPhour;
	int BPmin;
	int BPsec;
	int iBegin;
	int iEnd;
	int iValue;
	int iValueBegin;
	int iValueEnd;
	std::string sString;
	std::string sFormat;
	std::string sCommand;
	std::string sValue;
	time_t tCurDate;
	tm *tmCurDate;

	// reset error code
	iBPError = 0;

	// verify input string
	if (BPdate == "")
		iBPError = 8;

	// verify format
	if (sBP_Format == "")
		iBPError = 9;


	if (iBPError == 0)
	{
		BPyear = 0;
		BPmonth = 0;
		BPday = 0;
		BPhour = 0;
		BPmin = 0;
		BPsec = 0;

		// explore input string :
		sFormat = BP_Date::sBP_Format;

		iBegin = 0;
		iValueBegin = 0;
		while (iBegin < sFormat.length() && iValueBegin < BPdate.length())
		{
			// search for date/time command into format string
			sString = "";
			iEnd = iBegin;
			while (sFormat [iEnd] != 'd' &&
				   sFormat [iEnd] != 'm' &&
				   sFormat [iEnd] != 'y' &&
				   sFormat [iEnd] != 'h' &&
				   sFormat [iEnd] != 'n' &&
	   			   sFormat [iEnd] != 's' &&
				   iEnd < sFormat.length())
			{
				if (sFormat [iEnd] == '!')
					iEnd++;
				sString += sFormat [iEnd];
				iEnd++;
			}


			// search full command
			iBegin = iEnd;
			iEnd++;
			while (sFormat [iEnd] == sFormat [iBegin] &&
				   iEnd < sFormat.length())
			{
				iEnd++;
			}

			// store command
			sCommand = sFormat.substr(iBegin, iEnd - iBegin);

			// search numeric value into input string
			iValueEnd = iValueBegin;
			while ((BPdate [iValueEnd] < '0' || BPdate [iValueEnd] > '9')
				   && iValueEnd < BPdate.length())
			{
				iValueEnd++;
			}

			// retreive full numeric value
			iValueBegin = iValueEnd;
			iValueEnd++;
			while ((BPdate [iValueEnd] >= '0' && BPdate [iValueEnd] <= '9')
				   && iValueEnd < BPdate.length())
			{
				iValueEnd++;
			}

			// store value into integer
			sValue = BPdate.substr(iValueBegin, iValueEnd - iValueBegin);
			std::istringstream sStream (sValue);
			sStream >> iValue;

			// interpret command
			if (sCommand == "d" || sCommand == "dd")
				BPday = iValue;
			else if (sCommand == "m" || sCommand == "mm")
				BPmonth = iValue;
			else if (sCommand == "yy")
			{
				if (iValue < 38)
					BPyear = iValue + 2000;
				else if (iValue < 500)
					BPyear = iValue + 1900;
				else
					BPyear = iValue;
			}
			else if (sCommand == "yyyy")
				BPyear = iValue;
			else if (sCommand == "h" || sCommand == "hh")
				BPhour = iValue;
			else if (sCommand == "n" || sCommand == "nn")
				BPmin = iValue;
			else if (sCommand == "s" || sCommand == "ss")
				BPsec = iValue;

			iBegin = iEnd;
			iValueBegin = iValueEnd;

		}

		// date is not null
		bBP_Null = false;


		// get current date & time
		time (&tCurDate);
		tmCurDate = localtime (&tCurDate);

		// set default year
		if (BPyear == 0)
			BPyear = tmCurDate->tm_year + 1900;

		// set default month
		if (BPmonth == 0)
			BPmonth = tmCurDate->tm_mon + 1;

		// store date and/or time
		BP_Date::setdatetime (BPyear, BPmonth, BPday, BPhour, BPmin, BPsec);

	}
	else
	{
		// set date to null
		BP_Date::setnull ();
	}

}



//-----------------------------------------------------------------
//!    BP_Date::geterror : return error code                      !
//-----------------------------------------------------------------
//! return : error code or 0 if none                              !
//-----------------------------------------------------------------
int BP_Date::geterror (void)
{

	return iBPError;

}



//-----------------------------------------------------------------
//!    BP_Date::str : return date and time into a string          !
//!                        depending on format                    !
//-----------------------------------------------------------------
//! BPformat : temporary format, default value is ""              !
//!                                                               !
//! return : string or "" if date is null                         !
//-----------------------------------------------------------------
//! use BP_Date::setformat before BP_Date::str if BPformat == ""  !
//! if error occurs, the return string = "" and                   !
//!     BP_Date::geterror() return error code                     !
//-----------------------------------------------------------------
std::string BP_Date::str (const std::string BPformat)
{
	std::ostringstream os;

	tm *tmBPStruct;
	time_t tBPtime;
	int iBegin;
	int iEnd;
	std::string sFormat;
	std::string sString;
	std::string sCommand;


	// reset error code
	iBPError = 0;

	// return "" if date is null
	if (bBP_Null == true)
		return "";
	
	// transfert datetime
	tBPtime = tBP_Date;
	
	// use user format if not null
	if (BPformat == "")
		sFormat = sBP_Format;
	else
		sFormat = BPformat;

	// date too big or too small
	if (tBPtime == (time_t)-1)
	{
		iBPError = 7;
		return "";
	}

	// date is empty
	if (tBPtime == (time_t)0)
	{
		iBPError = 8;
		return "";
	}

	// format is empty
	if (sFormat == "")
	{
		iBPError = 9;
		return "";
	}

	// convert into time structure
	tmBPStruct = localtime(&tBPtime);


	// construct output stringstream
	iBegin = 0;
	while (iBegin < sFormat.length())
	{
		// search for date/time command
		sString = "";
		iEnd = iBegin;
		while (sFormat [iEnd] != 'd' &&
			   sFormat [iEnd] != 'm' &&
			   sFormat [iEnd] != 'y' &&
			   sFormat [iEnd] != 'h' &&
			   sFormat [iEnd] != 'n' &&
   			   sFormat [iEnd] != 's' &&
			   iEnd < sFormat.length())
		{
			if (sFormat [iEnd] == '!')
				iEnd++;
			sString += sFormat [iEnd];
			iEnd++;
		}

		// copy leading character
		if (iEnd > iBegin)
			os << sString;

		// search full command
		iBegin = iEnd;
		iEnd++;
		while (sFormat[iEnd] == sFormat[iBegin] &&
			   iEnd < sFormat.length())
		{
			iEnd++;
		}

		// interpret command
		sCommand = sFormat.substr(iBegin, iEnd - iBegin);

		// day
		if (sCommand == "d")
			os << tmBPStruct->tm_mday;
		else if (sCommand == "dd")
		{
			if (tmBPStruct->tm_mday < 10)
				os << "0";
			os << tmBPStruct->tm_mday;
		}
		// month
		else if (sCommand == "m")
			os << tmBPStruct->tm_mon + 1;
		else if (sCommand == "mm")
		{
			if (tmBPStruct->tm_mon < 9)
				os << "0";
			os << tmBPStruct->tm_mon + 1;
		}
		// year
		else if (sCommand == "yy")
		{
			if (tmBPStruct->tm_year < 100)
				os << tmBPStruct->tm_year;
			else
			{
				if (tmBPStruct->tm_year < 110)
					os << '0';
				os << tmBPStruct->tm_year - 100;
			}
		}
		else if (sCommand == "yyyy")
			os << tmBPStruct->tm_year + 1900;
		// hour
		else if (sCommand == "h")
			os << tmBPStruct->tm_hour;
		else if (sCommand == "hh")
		{
			if (tmBPStruct->tm_hour < 10)
				os << "0";
			os << tmBPStruct->tm_hour;
		}
		// minute
		else if (sCommand == "n")
			os << tmBPStruct->tm_min;
		else if (sCommand == "nn")
		{
			if (tmBPStruct->tm_min < 10)
				os << "0";
			os << tmBPStruct->tm_min;
		}
		// second
		else if (sCommand == "s")
			os << tmBPStruct->tm_sec;
		else if (sCommand == "ss")
		{
			if (tmBPStruct->tm_sec < 10)
				os << "0";
			os << tmBPStruct->tm_sec;
		}

		iBegin = iEnd;
	}

	return os.str();

}



//-----------------------------------------------------------------
//!    BP_Date::setformat : set the format for the date           !
//!                         use by BP_Date::str                   !
//-----------------------------------------------------------------
//! BPformat : string containing description of the format        !
//!                                                               !
//! command :                                                     !
//! d  = number of the day (ie : 1)                               !
//! dd = number of the day, 2 chars (ie : 01)                     !
//! m  = number of the month (ie : 2)                             !
//! mm = number of the month, 2 chars (ie : 02)                   !
//! yy = number of the year, 2 chars (ie : 99)                    !
//! yyyy = number of the year, 4 chars (ie : 1999)                !
//! h  = number of the hour (ie : 3)                              !
//! hh = number of the hour, 2 chars (ie : 03)                    !
//! n  = number of the minute (ie : 4)                            !
//! nn = number of the minute, 2 chars (ie : 04)                  !
//! s  = number of the second (ie : 5)                            !
//! ss = number of the second, 2 chars (ie : 05)                  !
//! !x = print the character x (ie : !d give "d")                 !
//! other = all the other character stay unmodified               !
//!                                                               !
//! samples :                                                     !
//! "dd/mm/yyyy hh:nn:ss" give "01/02/2004 08:09:01"              !
//! "m/d/yy h:n:s" give "12/31/99 1:2:15"                         !
//! "d/m/yy" give "31/12/04"                                      !
//! "hh:nn" give "08:01"                                          !
//! "tod!ay is : d mm yyyy" give "today is : 1 03 2004"           !
//!                                                               !
//! return : none                                                 !
//-----------------------------------------------------------------
//!     BP_Date::geterror() return error code                     !
//-----------------------------------------------------------------
BP_Date::setformat (const std::string BPformat)
{

	// reset error code
	iBPError = 0;

	// verify format
	if (BPformat == "")
	{
		iBPError = 9;
	}
	else
	{
		// store the format
		sBP_Format = BPformat;
	}

}



//-----------------------------------------------------------------
//!    BP_Date::getformat : return the format for the date        !
//-----------------------------------------------------------------
//! return : string containing the format in use                  !
//-----------------------------------------------------------------
std::string BP_Date::getformat (void)
{

	// reset error code
	iBPError = 0;

	return sBP_Format;

}



//-----------------------------------------------------------------
//!    BP_Date::day : return the day                              !
//-----------------------------------------------------------------
//! return : integer containing the day of the date (1-31)        !
//!           or -1 if null                                       !
//-----------------------------------------------------------------
int BP_Date::day (void)
{

	tm *tmBPStruct;

	// reset error code
	iBPError = 0;

	// if date is not null
	if (bBP_Null == true)
	{
		return -1;
	}
	else
	{
		// convert date into time structure
		tmBPStruct = localtime(&tBP_Date);

		return tmBPStruct->tm_mday;
	}

}



//-----------------------------------------------------------------
//!    BP_Date::month : return the month                          !
//-----------------------------------------------------------------
//! return : integer containing the month of the date (1-12)      !
//!           or -1 if null                                       !
//-----------------------------------------------------------------
int BP_Date::month (void)
{

	tm *tmBPStruct;

	// reset error code
	iBPError = 0;

	// if date is not null
	if (bBP_Null == true)
	{
		return -1;
	}
	else
	{
		// convert date into time structure
		tmBPStruct = localtime(&tBP_Date);

		return tmBPStruct->tm_mon + 1;
	}

}



//-----------------------------------------------------------------
//!    BP_Date::year : return the year                            !
//-----------------------------------------------------------------
//! return : integer containing the year of the date (1970-2038)  !
//!           or -1 if null                                       !
//-----------------------------------------------------------------
int BP_Date::year (void)
{

	tm *tmBPStruct;

	// reset error code
	iBPError = 0;

	// if date is not null
	if (bBP_Null == true)
	{
		return -1;
	}
	else
	{
		// convert date into time structure
		tmBPStruct = localtime(&tBP_Date);

		return tmBPStruct->tm_year + 1900;
	}

}



//-----------------------------------------------------------------
//!    BP_Date::hour : return the hour                            !
//-----------------------------------------------------------------
//! return : integer containing the hour of the date (0-23)       !
//!           or -1 if null                                       !
//-----------------------------------------------------------------
int BP_Date::hour (void)
{

	tm *tmBPStruct;

	// reset error code
	iBPError = 0;

	// if date is not null
	if (bBP_Null == true)
	{
		return -1;
	}
	else
	{
		// convert date into time structure
		tmBPStruct = localtime(&tBP_Date);

		return tmBPStruct->tm_hour;
	}

}



//-----------------------------------------------------------------
//!    BP_Date::minute : return the minute                        !
//-----------------------------------------------------------------
//! return : integer containing the minute of the date (0-59)     !
//!           or -1 if null                                       !
//-----------------------------------------------------------------
int BP_Date::minute (void)
{

	tm *tmBPStruct;

	// reset error code
	iBPError = 0;

	// if date is not null
	if (bBP_Null == true)
	{
		return -1;
	}
	else
	{
		// convert date into time structure
		tmBPStruct = localtime(&tBP_Date);

		return tmBPStruct->tm_min;
	}

}



//-----------------------------------------------------------------
//!    BP_Date::second : return the second                        !
//-----------------------------------------------------------------
//! return : integer containing the second of the date (0-59)     !
//!           or -1 if null                                       !
//-----------------------------------------------------------------
int BP_Date::second (void)
{

	tm *tmBPStruct;

	// reset error code
	iBPError = 0;

	// if date is not null
	if (bBP_Null == true)
	{
		return -1;
	}
	else
	{
		// convert date into time structure
		tmBPStruct = localtime(&tBP_Date);

		return tmBPStruct->tm_sec;
	}

}



//-----------------------------------------------------------------
//!    BP_Date::settimet : set the time from time_t container     !
//-----------------------------------------------------------------
//! return : none                                                 !
//-----------------------------------------------------------------
BP_Date::settimet (time_t BPtimet)
{

	// reset error code
	iBPError = 0;

	tBP_Date = BPtimet;

}



//-----------------------------------------------------------------
//!    BP_Date::gettimet : return the time in time_t container    !
//-----------------------------------------------------------------
//! return : time_t containing the date                           !
//-----------------------------------------------------------------
time_t BP_Date::gettimet (void)
{

	// reset error code
	iBPError = 0;

	return tBP_Date;

}



//-----------------------------------------------------------------
//!    BP_Date::isnull : return true if date is null              !
//!                           else return false                   !
//-----------------------------------------------------------------
//! return : bool                                                 !
//-----------------------------------------------------------------
bool BP_Date::isnull (void)
{

	// reset error code - keep error code
//	iBPError = 0;

	if (bBP_Null == true)
		return true;
	else
		return false;

}



//-----------------------------------------------------------------
//!    BP_Date::setnull : set date to null value                  !
//-----------------------------------------------------------------
//! return : none                                                 !
//-----------------------------------------------------------------
BP_Date::setnull (void)
{

	// reset error code - no keep error code
//	iBPError = 0;

	tBP_Date = 0;
	bBP_Null = true;

}



//-----------------------------------------------------------------
//!    BP_Date::setnow : set date to actual date and time         !
//-----------------------------------------------------------------
//! return : none                                                 !
//-----------------------------------------------------------------
BP_Date::setnow (void)
{

	// reset error code
	iBPError = 0;

	time (&tBP_Date);
	bBP_Null = false;

}


